/*
	list.C

	List all files in specified or current directory.

	Written by: Geoff Friesen
	Written in: Turbo C 1.5

	This program is not copywritten and may be freely
	integrated into your own work.
*/

#include <alloc.H>
#include <dir.H>
#include <dos.H>
#include <process.H>
#include <stdio.H>
#include <stdlib.H>
#include <string.H>

void fde (struct ffblk *, char *);

void main (int argc, char **argv)
{
   int done;
   struct ffblk de;
   char buffer [42], *path;

   if (_osmajor < 2 || _osmajor == 2 && _osminor < 1)
   {
       fprintf (stderr, "list: DOS 2.1 or higher required\n");
       exit (1);
   }

   if (argc > 2)
   {
      fprintf (stderr, "list:  invalid number of arguments\n");
      fprintf (stderr, "usage: list [path]\n");
      exit (1);
   }

   if (argc == 2)
   {
      if (*(argv [1] + strlen (argv [1]) - 1) == '\\')
      {
         path = (char *) malloc (strlen (argv [1]) + 4);
         strcpy (path, argv [1]);
         strcat (path, "*.*");
      }
      else
      {
	 path = (char *) malloc (strlen (argv [1]) + 1);
	 strcpy (path, argv [1]);
      }
   }
   else
   {
      path = (char *) malloc (4);
      strcpy (path, "*.*");
   }

   printf ("\nDirectory listing of %s\n\n", strupr (path));

   done = findfirst (path, &de, FA_DIREC | FA_HIDDEN | FA_SYSTEM);

   while (!done)
   {
      fde (&de, buffer);
      puts (buffer);
      done = findnext (&de);
   }

   free (path);
}

/* -------------------------------------------- */
/* void fde (struct ffblk *de, char *buffer);   */
/*                                              */
/* Take the ffblk directory entry pointed to by */
/* de and format it into an ASCIZ string.  The  */
/* buffer must be at least 42 bytes long.  This */
/* includes the terminating null.               */
/* -------------------------------------------- */

void fde (struct ffblk *de, char *buffer)
{
   int c, tmp;
   size_t index, length;
   char *ptr, tmpbuf [12];

/* 0. Prepare buffer */

   memset (buffer, ' ', 41);
   buffer [41] = '\0';

/* 1. Process directory entry name */

   length = strlen (de->ff_name);

   if (de->ff_name [0] == '.')
      memcpy (buffer, de->ff_name, length);
   else
   {
      if (!(de->ff_attrib & FA_DIREC))
         strlwr (de->ff_name);

      if (!(ptr = (char *) memchr (de->ff_name, '.', length)))
	 memcpy (buffer, de->ff_name, length);
      else
      {
	 memcpy (buffer, de->ff_name, length - strlen (ptr));
	 memcpy (buffer+9, ptr+1, strlen (ptr+1));
      }
   }

   if (de->ff_attrib & FA_LABEL)
      return;

   index = 13;

/* 2. Process file size */

   if (!(de->ff_attrib & FA_DIREC))
   {
      ltoa (de->ff_fsize, tmpbuf, 10);
      length = strlen (tmpbuf);
      memcpy (buffer+index+10-length, tmpbuf, length);
   }
   else
      memcpy (buffer+index+4, "[DIR]", 5);

   index += 12;

/* 3. Process directory entry date */

   itoa ((de->ff_fdate & 0x01e0) >> 5, tmpbuf, 10);
   strcat (tmpbuf, "-");
   if ((length = strlen (tmpbuf)) == 2)
      index++;

   memcpy (buffer+index, tmpbuf, length);

   index += length;

   itoa (de->ff_fdate & 0x001f, tmpbuf, 10);
   strcat (tmpbuf, "-");
   if ((length = strlen (tmpbuf)) == 2)
      buffer [index++] = '0';

   memcpy (buffer+index, tmpbuf, length);

   index += length;

   itoa (((de->ff_fdate & 0xfe00) >> 9) + 80, tmpbuf, 10);
   if ((length = strlen (tmpbuf)) == 1)
      index++;
   if (length == 3)
   {
      length--;
      memcpy (buffer+index, "00", 2);
   }
   else
      memcpy (buffer+index, tmpbuf, length);

   index += (length+2);

/* 4. Process directory entry time */

   tmp = (unsigned char) ((de->ff_ftime & 0xf800) >> 11);
   c = (tmp >= 12) ? 'p' : 'a';

   if (!tmp)
      tmp = 12;
   else
      if (tmp > 12)
	 tmp -= 12;

   itoa (tmp, tmpbuf, 10);
   strcat (tmpbuf, ":");
   if ((length = strlen (tmpbuf)) == 2)
      index++;

   memcpy (buffer+index, tmpbuf, length);

   index += length;

   tmp = (unsigned char) ((de->ff_ftime & 0x07e0) >> 5);

   itoa (tmp, tmpbuf, 10);
   if ((length = strlen (tmpbuf)) == 1)
      buffer [index++] = '0';

   memcpy (buffer+index, tmpbuf, length);

   index += length;

   buffer [index] = c;
}